'''
database.py - Database connection module.

(c) 2005 Jan ONDREJ (SAL) <ondrejj(at)salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

'''

from avlib import debug,safe

classes=['MySQLdb','sqlite','pg','pgdb','psycopg']

class dbconn:
  name='connection_template'
  params=('host','port','dbname','dbuser','dbpasswd')
  def __init__(self,db=None,params=[]):
      self.DB=db
      self.PARAMS=params
  def __del__(self):
      try:
        self.DBC.commit()
        self.DBC.close()
      except:
        pass
  def execute(self,sql_cmd,commit=True):
      try:
        cur=self.DBC.cursor()
      except:
        self.connect()
        cur=self.DBC.cursor()
      cur.execute(sql_cmd)
      self.rowcount=cur.rowcount
      if commit:
        self.DBC.commit()
      return cur
  def query(self,sql_cmd):
      return self.execute(sql_cmd,False).fetchall()
  def commit(self):
      self.DBC.commit()
  def quote(self,s):
      return s.replace("'","''").replace('\\','\\\\')
  def close(self):
      self.DBC.close()

class MySQLdb(dbconn):
  name='MySQLdb'
  params=('host','port','dbname','dbuser','dbpasswd')
  def __init__(self,params):
      try:
        import MySQLdb
      except ImportError,es:
        sys.exit("Error importing MySQLdb module for python: "+str(es))
      dbconn.__init__(self,MySQLdb,params)
      # load required modules only
      try: self.connect()
      except: pass
  def connect(self):
      self.DBC=self.DB.connect(
        host=self.PARAMS.get('host'),
        port=self.PARAMS.get('port'),
        db=self.PARAMS.get('dbname'),
        user=self.PARAMS.get('dbuser'),
        passwd=self.PARAMS.get('dbpasswd'),
        connect_timeout=5
      )
  def quote(self,s):
      return self.DB.escape_string(s)

class sqlite(dbconn):
  name='sqlite'
  params=('dbname')
  def __init__(self,params):
      try:
        import sqlite
      except ImportError,es:
        sys.exit("Error importing SQLite module for python: "+str(es))
      dbconn.__init__(self,sqlite,params)
      #self.connect()
  def connect(self):
      self.DBC=self.DB.connect(
        safe.fn(self.PARAMS.get('dbname'))
      )

class pg(dbconn):
  name='pg'
  params=('host','port','dbname','dbuser','dbpasswd')
  def __init__(self,params):
      try:
        import pg
      except ImportError,es:
        sys.exit("Error importing PostgreSQL module for python: "+str(es))
      dbconn.__init__(self,pg,params)
      #self.connect()
  def connect(self):
      self.DBC=self.DB.connect(
        dbname=self.PARAMS.get('dbname'),
        host=self.PARAMS.get('host'),
        port=self.PARAMS.get('port'),
        user=self.PARAMS.get('dbuser'),
        passwd=self.PARAMS.get('dbpasswd')
      )

class pgdb(dbconn):
  name='pgdb'
  params=('host','port','dbname','dbuser','dbpasswd')
  def __init__(self,params):
      try:
        import pgdb
      except ImportError,es:
        sys.exit("Error importing PostgreSQL module for python: "+str(es))
      dbconn.__init__(self,pgdb,params)
      #self.connect()
  def connect(self):
      self.DBC=self.DB.connect(
        user=self.PARAMS.get('dbuser'),
        password=self.PARAMS.get('dbpasswd'),
        host="%s:%s" % (self.PARAMS.get('host'),self.PARAMS.get('port')),
        database=self.PARAMS.get('dbname')
      )

class psycopg(dbconn):
  name='psycopg'
  params=('host','port','dbname','dbuser','dbpasswd')
  def __init__(self,params):
      try:
        import pgdb
      except ImportError,es:
        sys.exit("Error importing psycopg module for python: "+str(es))
      dbconn.__init__(self,psycopg,params)
      #self.connect()
  def connect(self):
      self.DBC=self.DB.connect(
        "host=%s:%d dbname=%s user=%s password=%s" % (
          self.PARAMS.get('host'),
          self.PARAMS.get('port') or 5432,
          self.PARAMS.get('dbname'),
          self.PARAMS.get('dbuser'),
          self.PARAMS.get('dbpasswd')
        )
      )
